@@ -298,6 +298,8 @@ struct lyxp_set {
     uint32_t ctx_size;          /**< Position of the last node at the time the node was examined. */
     ly_bool non_child_axis;     /**< Whether any node change was performed on a non-child axis. */
 
+    ly_bool not_found;          /**< Set if a node is not found and it is considered an error. */
+
     /* general context */
     struct ly_ctx *ctx;                     /**< General context for logging. */
 
@@ -327,6 +329,8 @@ const char *lyxp_token2str(enum lyxp_token tok);
  * @brief Evaluate an XPath expression on data. Be careful when using this function, the result can often
  * be confusing without thorough understanding of XPath evaluation rules defined in RFC 7950.
  *
+ * Traverses (valid) opaque nodes in the evaluation.
+ *
  * @param[in] ctx libyang context to use.
  * @param[in] exp Parsed XPath expression to be evaluated.
  * @param[in] cur_mod Current module for the expression (where it was "instantiated").
@@ -421,6 +425,20 @@ ly_bool lyxp_set_scnode_contains(struct lyxp_set *set, const struct lysc_node *n
  */
 void lyxp_set_scnode_merge(struct lyxp_set *set1, struct lyxp_set *set2);
 
+/**
+ * @brief Insert schema node into set.
+ *
+ * @param[in] set Set to insert into.
+ * @param[in] node Node to insert.
+ * @param[in] node_type Node type of @p node.
+ * @param[in] axis Axis that @p node was reached on.
+ * @param[out] index_p Optional pointer to store the index of the inserted @p node.
+ * @return LY_SUCCESS on success.
+ * @return LY_EMEM on memory allocation failure.
+ */
+LY_ERR lyxp_set_scnode_insert_node(struct lyxp_set *set, const struct lysc_node *node, enum lyxp_node_type node_type,
+        enum lyxp_axis axis, uint32_t *index_p);
+
 /**
  * @brief Parse an XPath expression into a structure of tokens.
  *        Logs directly.
